/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet;

import javax.swing.JDialog;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JFrame;
import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.Box;
import javax.swing.Action;
import javax.swing.AbstractAction;
import javax.swing.ActionMap;
import java.awt.Dimension;
import java.awt.Color;
import java.awt.BorderLayout;
import java.awt.Container;
import java.awt.event.ActionEvent;

/**
 * Dialog box <i>About </i>.
 *
 * @author Adam Domaski
 * @version 0.1 11/02/2004
 */
public class AboutBox extends JetDialog
{
	/** "OK" Button. */
	private JButton btn;

	/** Resource. */
	private JetResources resource;

	/**
	 * Creates dialog box.
	 *
	 * @param window A parent window.
	 */
	public AboutBox(JFrame parent)
	{
		super(parent);
		
		resource = JetResources.getInstance();
		setTitle( resource.translate("AboutBox.title") );

		getContentPane().add( prepare() );
		getRootPane().setDefaultButton(btn);

		setDefaultCloseOperation(DISPOSE_ON_CLOSE);

		pack();
		setResizable(false);
		centerDialog();
		setVisible(true);
	}

	/**
	 * Metoda przygotowujca panel okna dialogowego
	 *
	 * @return Panel z wszystkimi kontrolkami.
	 */
	private Container prepare()
	{
		JLabel lbl;

		JPanel panel = new JPanel();
		panel.setBorder( BorderFactory.createEmptyBorder(10, 10, 10, 10) );
		panel.setLayout(new BorderLayout(0, 10));

		JPanel pp = new JPanel();
		pp.setBorder(BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(5, 5, 5, 5)));
		pp.setLayout( new BoxLayout(pp, BoxLayout.Y_AXIS) );
		// Dodaj autora:
		lbl = new JLabel( resource.translate("AboutBox.author") );
		lbl.setPreferredSize(new Dimension(160, 16));
		pp.add(lbl);
		
		lbl = new JLabel("Adam Doma\u0144ski");
		lbl.setForeground(Color.BLUE);
		pp.add(lbl);
		pp.add(Box.createRigidArea(new Dimension(0, 10)));

		lbl = new JLabel( resource.translate("AboutBox.version") );
		pp.add(lbl);

		lbl = new JLabel("1.0beta1 29/06/2004");
		lbl.setForeground(Color.BLUE);
		pp.add(lbl);

		panel.add(pp);

		Action clAction = new CloseAction();

		ActionMap actionMap = getLayeredPane().getActionMap();
		actionMap.put("close-it", clAction);
		actionMap.put("enter-it", clAction);

		btn = new JButton( resource.translate("AboutBox.close") );
		btn.addActionListener(clAction);
		panel.add(btn, BorderLayout.SOUTH);

		return panel;
	}

	class CloseAction extends AbstractAction
	{
		public void actionPerformed(ActionEvent ae)
		{
			setVisible(false);
		}
	}

}
