/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet;

import java.util.HashMap;
import java.util.Properties;
import java.util.Map;
import java.util.Iterator;
import java.util.StringTokenizer;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.awt.Color;
import javax.swing.UIManager;

/**
 * The configuration of <b>jET</b>.
 * <p>It has similar features as <code>java.util.Properties</code>
 * class. It can read and write properties to files. The extension is
 * in that it can save values other than strings.</p>
 *
 * <p>Implementation bases on hashed map.</p>
 *
 * @author Adam Domaski
 * @version 0.1 11/03/2004
 */
public class Configuration extends HashMap
{
	/**
	 * Constructs configuration object.
	 */
	public Configuration()
	{
		super();
	}

	// ----- Loading and storing ---------

	/**
	 * Loads configuration from file.
	 * If file doesn't exist or there is no
	 * key, the configuration get defaults.
	 */
	public void load()
	{
		Properties prop = new Properties();

		try
		{
			FileInputStream fileIn = new FileInputStream(FILENAME);
			prop.load(fileIn);
			fileIn.close();
		}
		catch (IOException ioe) {}

		loadProperties(prop);
	}

	/**
	 * Stores current configuration to the file.
	 */
	public void store()
	{
		Properties prop = storeProperties();

		try
		{
			FileOutputStream fileOut = new FileOutputStream("jet.conf");		
			prop.store(fileOut, null);
			fileOut.close();
		}
		catch (IOException ioe) {}
	}

	/**
	 * Gets key value pairs from properties loaded from file and
	 * puts it to configuration map.
	 * If some properties were not found in the file, gives them
	 * default values.
	 *
	 * <p>This method is invoked inside <code>load()</code> method after reading
	 * file.</p>
	 *
	 * @param prop Properties loaded from file.
	 */
	protected void loadProperties(Properties prop)
	{
		Boolean b;
		Integer i;
		Double d;

		b = Boolean.valueOf( prop.getProperty("EditorGrid", "true") );
		put("EditorGrid", b);

		b = Boolean.valueOf( prop.getProperty("EditorRulers", "true") );
		put("EditorRulers", b);

		i = Integer.valueOf( prop.getProperty("WindowPosX", "0") );
		put("WindowPosX", i);

		i = Integer.valueOf( prop.getProperty("WindowPosY", "0") );
		put("WindowPosY", i);

		i = Integer.valueOf( prop.getProperty("WindowWidth", "680") );
		put("WindowWidth", i);

		i = Integer.valueOf( prop.getProperty("WindowHeight", "420") );
		put("WindowHeight", i);

		b = Boolean.valueOf( prop.getProperty("TrackWiden", "false") );
		put("TrackWiden", b);

		d = Double.valueOf( prop.getProperty("AdmissibleUnbalancedAcceleration", "0.6") );
		put("AdmissibleUnbalancedAcceleration", d);

		String s;
		s = prop.getProperty("CurrentDirectory");
		put("CurrentDirectory", s);

		s = prop.getProperty("TextureDirectory");
		put("TextureDirectory", s);

		i = Integer.valueOf( prop.getProperty("PalettePosX", "0") );
		put("PalettePosX", i);

		i = Integer.valueOf( prop.getProperty("PalettePosY", "0") );
		put("PalettePosY", i);

		i = Integer.valueOf( prop.getProperty("PaletteWidth", "180") );
		put("PaletteWidth", i);

		i = Integer.valueOf( prop.getProperty("PaletteHeight", "380") );
		put("PaletteHeight", i);

		// Colors
		s = prop.getProperty( "EditorBackground" );
		put("EditorBackground", 
			decodeColor(s, UIManager.getColor("ToggleButton.select")));

		s = prop.getProperty( "EditorGridColor" );
		put("EditorGridColor", 
			decodeColor(s, UIManager.getColor("ComboBox.background")));

		s = prop.getProperty( "EditorTrackColor" );
		put("EditorTrackColor",
			decodeColor(s, Color.blue));

		s = prop.getProperty( "EditorSwitchColor" );
		put("EditorSwitchColor",
			decodeColor(s, Color.green));

		i = Integer.valueOf( prop.getProperty("EditorScrollIncrement", "20") );
		put("EditorScrollIncrement", i);

		s = prop.getProperty( "EditorTrackNamesColor" );
		put( "EditorTrackNamesColor", decodeColor(s, new Color(255, 235, 52)) );

		b = Boolean.valueOf( prop.getProperty("EditorTrackNames", "true") );
		put("EditorTrackNames", b);

		// Custom straight properties
		d = Double.valueOf( prop.getProperty("CustomStraightProperties.length", "100.0") );
		put("CustomStraightProperties.length", d);
		d = Double.valueOf( prop.getProperty("CustomStraightProperties.width", "1.435") );
		put("CustomStraightProperties.width", d);
		d = Double.valueOf( prop.getProperty("CustomStraightProperties.friction", "0.25") );
		put("CustomStraightProperties.friction", d);
		d = Double.valueOf( prop.getProperty("CustomStraightProperties.tap", "20.0") );
		put("CustomStraightProperties.tap", d);
		i = Integer.valueOf( prop.getProperty("CustomStraightProperties.quality", "20") );
		put("CustomStraightProperties.quality", i);
		i = Integer.valueOf( prop.getProperty("CustomStraightProperties.damage", "0") );
		put("CustomStraightProperties.damage", i);
		s = prop.getProperty("CustomStraightProperties.enviroment", "flat");
		put("CustomStraightProperties.enviroment", s);
		b = Boolean.valueOf( prop.getProperty("CustomStraightProperties.visibility", "true") );
		put("CustomStraightProperties.visibility", b);
		s = prop.getProperty("CustomStraightProperties.texture", "Rail_screw_used1");
		put("CustomStraightProperties.texture", s);
		d = Double.valueOf( prop.getProperty("CustomStraightProperties.texLength", "4.0") );
		put("CustomStraightProperties.texLength", d);
		s = prop.getProperty("CustomStraightProperties.secondTexture", "TpBpS-new2.tex");
		put("CustomStraightProperties.secondTexture", s);
		d = Double.valueOf( prop.getProperty("CustomStraightProperties.texWidth", "0.5") );
		put("CustomStraightProperties.texWidth", d);
		d = Double.valueOf( prop.getProperty("CustomStraightProperties.texHeight", "0.2") );
		put("CustomStraightProperties.texHeight", d);
		d = Double.valueOf( prop.getProperty("CustomStraightProperties.texSlope", "1.1") );
		put("CustomStraightProperties.texSlope", d);
		d = Double.valueOf( prop.getProperty("CustomStraightProperties.roll1", "0.0") );
		put("CustomStraightProperties.roll1", d);
		d = Double.valueOf( prop.getProperty("CustomStraightProperties.roll2", "0.0") );
		put("CustomStraightProperties.roll2", d);

		// Custom arc properties
		d = Double.valueOf( prop.getProperty("CustomArcProperties.length", "100.0") );
		put("CustomArcProperties.length", d);
		d = Double.valueOf( prop.getProperty("CustomArcProperties.radius", "300.0") );
		put("CustomArcProperties.radius", d);
		d = Double.valueOf( prop.getProperty("CustomArcProperties.width", "1.435") );
		put("CustomArcProperties.width", d);
		d = Double.valueOf( prop.getProperty("CustomArcProperties.friction", "0.25") );
		put("CustomArcProperties.friction", d);
		d = Double.valueOf( prop.getProperty("CustomArcProperties.tap", "20.0") );
		put("CustomArcProperties.tap", d);
		i = Integer.valueOf( prop.getProperty("CustomArcProperties.quality", "20") );
		put("CustomArcProperties.quality", i);
		i = Integer.valueOf( prop.getProperty("CustomArcProperties.damage", "0") );
		put("CustomArcProperties.damage", i);
		s = prop.getProperty("CustomArcProperties.enviroment", "flat");
		put("CustomArcProperties.enviroment", s);
		b = Boolean.valueOf( prop.getProperty("CustomArcProperties.visibility", "true") );
		put("CustomArcProperties.visibility", b);
		s = prop.getProperty("CustomArcProperties.texture", "Rail_screw_used1");
		put("CustomArcProperties.texture", s);
		d = Double.valueOf( prop.getProperty("CustomArcProperties.texLength", "4.0") );
		put("CustomArcProperties.texLength", d);
		s = prop.getProperty("CustomArcProperties.secondTexture", "TpBpS-new2.tex");
		put("CustomArcProperties.secondTexture", s);
		d = Double.valueOf( prop.getProperty("CustomArcProperties.texWidth", "0.5") );
		put("CustomArcProperties.texWidth", d);
		d = Double.valueOf( prop.getProperty("CustomArcProperties.texHeight", "0.2") );
		put("CustomArcProperties.texHeight", d);
		d = Double.valueOf( prop.getProperty("CustomArcProperties.texSlope", "1.1") );
		put("CustomArcProperties.texSlope", d);
		d = Double.valueOf( prop.getProperty("CustomArcProperties.roll1", "0.0") );
		put("CustomArcProperties.roll1", d);
		d = Double.valueOf( prop.getProperty("CustomArcProperties.roll2", "0.0") );
		put("CustomArcProperties.roll2", d);
		b = Boolean.valueOf( prop.getProperty("CustomArcProperties.right", "true") );
		put("CustomArcProperties.right", b);

		// Custom switch properties
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.length1", "27.138") );
		put("CustomSwitchProperties.length1", d);
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.radius1", "0.0") );
		put("CustomSwitchProperties.radius1", d);
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.length2", "21.046") );
		put("CustomSwitchProperties.length2", d);
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.radius2", "190.0") );
		put("CustomSwitchProperties.radius2", d);
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.width", "1.435") );
		put("CustomSwitchProperties.width", d);
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.friction", "0.25") );
		put("CustomSwitchProperties.friction", d);
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.tap", "20.0") );
		put("CustomSwitchProperties.tap", d);
		i = Integer.valueOf( prop.getProperty("CustomSwitchProperties.quality", "20") );
		put("CustomSwitchProperties.quality", i);
		i = Integer.valueOf( prop.getProperty("CustomSwitchProperties.damage", "0") );
		put("CustomSwitchProperties.damage", i);
		s = prop.getProperty("CustomSwitchProperties.enviroment", "flat");
		put("CustomSwitchProperties.enviroment", s);
		b = Boolean.valueOf( prop.getProperty("CustomSwitchProperties.visibility", "true") );
		put("CustomSwitchProperties.visibility", b);
		s = prop.getProperty("CustomSwitchProperties.texture", "Rail_screw_used1");
		put("CustomSwitchProperties.texture", s);
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.texLength", "4.0") );
		put("CustomSwitchProperties.texLength", d);
		s = prop.getProperty("CustomSwitchProperties.secondTexture", "Rail_screw_used1");
		put("CustomSwitchProperties.secondTexture", s);
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.texWidth", "0.5") );
		put("CustomSwitchProperties.texWidth", d);
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.texHeight", "0.2") );
		put("CustomSwitchProperties.texHeight", d);
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.texSlope", "1.1") );
		put("CustomSwitchProperties.texSlope", d);
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.roll1", "0.0") );
		put("CustomSwitchProperties.roll1", d);
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.roll2", "0.0") );
		put("CustomSwitchProperties.roll2", d);
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.rollS", "-0.1") );
		put("CustomSwitchProperties.rollS", d);
		d = Double.valueOf( prop.getProperty("CustomSwitchProperties.roll3", "0.1") );
		put("CustomSwitchProperties.roll3", d);
		b = Boolean.valueOf( prop.getProperty("CustomSwitchProperties.right", "true") );
		put("CustomSwitchProperties.right", b);
		i = Integer.valueOf( prop.getProperty("CustomSwitchProperties.handler", "1") );
		put("CustomSwitchProperties.handler", i);
	}

	/**
	 * Stores key, value pairs from configuration to properties, which
	 * are going to be stored in file.
	 * <p>It is invoked inside <code>store()</code> method before writing to
	 * configuration file.</p>
	 *
	 * @return Properties with copied key value pairs from configuration.
	 */
	protected Properties storeProperties()
	{
		Properties prop = new Properties();

		Iterator i = entrySet().iterator();
		while (i.hasNext())
		{
			Map.Entry me = (Map.Entry) i.next();

			Object obj = me.getValue();
			if (obj != null)
			{
				if (obj instanceof Color)
				{
					prop.setProperty(me.getKey().toString(), encodeColor((Color) obj));
				}
				else
				{
					prop.setProperty(me.getKey().toString(), obj.toString());
				}
			}
		}

		return prop;
	}

	// ------------- Putting -------------

	/**
	 * Puts a integer to configuration.
	 *
	 * @param str Option name.
	 * @param num Integer number.
	 * @return Previous value of this key.
	 */
	public int put(String str, int num)
	{
		Integer i = (Integer) put(str, new Integer(num));
		return i.intValue();
	}

	/**
	 * Puts a boolean to configuration.
	 *
	 * @param str Option name.
	 * @param b Boolean value.
	 * @return Previous value of this key.
	 */
	public boolean put(String str, boolean b)
	{
		Boolean bl = (Boolean) put(str, new Boolean(b));
		return bl.booleanValue();
	}

	// ------------- Getting -------------

	/**
	 * Gets integer value.
	 *
	 * @param str Option name.
	 * @return The integer value of this key.
	 * @exception ClassCastException Method may throw a class cast exception
	 * when the key does not match integer value.	 
	 */
	public int getInt(String str)
	{
		Integer num = (Integer) get(str);
		return num.intValue();
	}

	/**
	 * Gets boolean value.
	 *
	 * @param str Option name.
	 * @return The boolean value of this key.
	 * @exception ClassCastException Method may throw a class cast exception
	 * when the key does not match boolean value.	 
	 */
	public boolean getBoolean(String str)
	{
		Boolean b = (Boolean) get(str);
		return b.booleanValue();
	}

	/**
	 * Gets double value.
	 *
	 * @param str Option name.
	 * @return The double value of given key.
	 * @exception ClassCastException Method may throw a class cast exception
	 * when the key does not match boolean value.	 
	 */
	public double getDouble(String str)
	{
		Double d = (Double) get(str);
		return d.doubleValue();
	}

	/*
	 * ------------- Colors --------------
	 */
	
	/**
	 * Encodes color to string.
	 */
	private String encodeColor(Color c)
	{
		StringBuffer sb = new StringBuffer();

		sb.append('(');
		sb.append( c.getRed() );
		sb.append(',');
		sb.append( c.getGreen() );
		sb.append(',');
		sb.append( c.getBlue() );
		sb.append(')');

		return sb.toString();
	}

	/**
	 * Decodes color for string.
	 */
	private Color decodeColor(String str, Color def)
	{
		Color c;

		if (str == null)
			return def;

		if (str.matches("^\\(\\d{1,3},\\d{1,3},\\d{1,3}\\)$"))
		{
			try
			{
				int i = 0;
				int[] array = new int[3];

				str = str.substring(1, str.length() - 1);
				StringTokenizer strZ = new StringTokenizer(str, ",");
				while (strZ.hasMoreTokens())
				{
					array[i] = Integer.parseInt( strZ.nextToken() );
					if (array[i] < 0 || array[i] > 255)
						return def;
					i++;
				}

				return new Color(array[0], array[1], array[2]);
			}
			catch (NumberFormatException nfe)
			{
				return def;
			}
		}
		else
		{
			c = def;
		}

		return c;
	}

	// ------------- Members -------------

	/**
	 * Configuration file name.
	 */
	public static final String FILENAME = "jet.conf";
}
