/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet;

import java.awt.*;
import java.awt.event.ActionEvent;
import javax.swing.*;
import javax.vecmath.Point3d;

/**
 * <i>Go to point</i> dialog.
 *
 * @author Adam Domaski
 * @version 0.1 16/02/2004
 */
public class GoToDialog extends JetDialog
{
	/**
	 * Creates dialog.
	 */
	public GoToDialog(Jet app)
	{
		super(app.getFrame());
		this.app = app;

		resource = JetResources.getInstance();
		setTitle( resource.translate("GoToDialog.title") );

		setResizable(false);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);

		initialize();		

		pack();
		centerDialog();
		okButton.requestFocus();
		setVisible(true);
	}

	private JButton okButton;

	private void initialize()
	{
		JPanel jPane = new JPanel();
		jPane.setLayout(new BorderLayout());

		JPanel jContainer = createContents();
		jPane.add(jContainer);

		JPanel jControlPanel = new JPanel();
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		jControlPanel.setLayout(gb);
		jControlPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		Action okAction = new OkAction();
		Action caAction = new CancelAction();

		ActionMap actionMap = getLayeredPane().getActionMap();
		actionMap.put("close-it", caAction);
		actionMap.put("enter-it", okAction);

		okButton = new JButton( resource.translate("GoToDialog.ok") );
		JButton caButton = new JButton( resource.translate("GoToDialog.cancel") );

		caButton.addActionListener(caAction);

		okButton.addActionListener(okAction);
		getRootPane().setDefaultButton(okButton);

		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.EAST;
		c.insets = new Insets(0, 0, 0, 5);
		gb.setConstraints(okButton, c);
		jControlPanel.add(okButton);

		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 0.0;
		c.insets = new Insets(0,0,0,0);
		gb.setConstraints(caButton, c);
		jControlPanel.add(caButton);

		normalize(okButton, caButton);

		jPane.add(jControlPanel, BorderLayout.SOUTH);

		getContentPane().add(jPane);
	}

	/**
	 * Creates contents.
	 */
	private JPanel createContents()
	{
		JLabel jLabel;
		JFormattedTextField jText;

		JPanel jPane = new JPanel();
		jPane.setName("GoToDialogPanel");
		jPane.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEmptyBorder(3, 3, 3, 3),
			BorderFactory.createCompoundBorder(
				BorderFactory.createEtchedBorder(),
				BorderFactory.createEmptyBorder(3, 3, 3, 3)) ) );

		jPane.setLayout(new GridLayout(3, 2, 0, 2));

		jLabel = new JLabel();
		jLabel.setName("XTextBoxLabel");
		jLabel.setText( resource.translate("GoToDialog.x") );
		jPane.add(jLabel);

		jText = JetDialog.createLongFormatTextBox();
		jText.setName("XTextBox");
		jText.setValue( new Double(0.0) );
		jText.setColumns(6);
		controls.putComponent(jText);
		jPane.add(jText);

		jLabel = new JLabel();
		jLabel.setName("YTextBoxLabel");
		jLabel.setText( resource.translate("GoToDialog.y") );
		jPane.add(jLabel);

		jText = JetDialog.createLongFormatTextBox();
		jText.setName("YTextBox");
		jText.setColumns(6);
		jText.setValue( new Double(0.0) );
		controls.putComponent(jText);
		jPane.add(jText);

		jLabel = new JLabel();
		jLabel.setName("ZTextBoxLabel");
		jLabel.setText( resource.translate("GoToDialog.z") );
		jPane.add(jLabel);

		jText = JetDialog.createLongFormatTextBox();
		jText.setName("ZTextBox");
		jText.setValue( new Double(0.0) );
		jText.setColumns(6);
		controls.putComponent(jText);
		jPane.add(jText);

		return jPane;
	}

	/**
	 * Gets entered point in model coordinates.
	 *
	 * @return <code>Null</code> if the user has cancelled dialog or
	 *   3-D point.
	 */
	public Point3d getPoint()
	{
		return returnedValue;
	}

	/**
	 * "OK" Action.
	 */
	class OkAction extends AbstractAction
	{
		public void actionPerformed(ActionEvent ae)
		{
			Component aComponent;
			Number d;

			Point3d point = new Point3d();
			aComponent = controls.getComponent("XTextBox");
			d = (Number) ((JFormattedTextField) aComponent).getValue();
			point.x = d.doubleValue();
			aComponent = controls.getComponent("YTextBox");
			d = (Number) ((JFormattedTextField) aComponent).getValue();
			point.y = d.doubleValue();
			aComponent = controls.getComponent("ZTextBox");
			d = (Number) ((JFormattedTextField) aComponent).getValue();
			point.z = d.doubleValue();

			returnedValue = point;

			dispose();
		}
	}

	class CancelAction extends AbstractAction
	{
		public void actionPerformed(ActionEvent ae)
		{
			dispose();
		}
	}

	/*
	 * --------------- Members ---------------
	 */
	
	/** Reference to application. */
	private Jet app;

	/** I18n resources. */
	private JetResources resource;

	/** Returned value. */
	private Point3d returnedValue = null;

	/** Components. */
	private ControlMap controls = new ControlMap();
}
