/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet;

import java.awt.Rectangle;
import java.awt.Insets;
import javax.swing.*;

/**
 * Toggle button for track palette.
 * It has custom model.
 *
 * @author Adam Domaski
 * @version 0.1 18/06/2004
 */
public class JPaletteButton extends JToggleButton
{
	/*
	 * ----------- Constants ------------
	 */

	/** String representing L&F class. */
    private static final String uiClassID = "PaletteButtonUI";

	/**
	 * Bound property name for value.
	 */
	public final static String VALUE_PROPERTY = "value";

	/*
	 * --------- Constructors -----------
	 */

	/**
	 * Creates new button with specified object and with proper palette button model.
	 *
	 * @param model Palette button model.
	 * @param obj Object of this object. It may be a track or a switch.
	 */
	public JPaletteButton(PaletteButtonModel model, Object obj)
	{
		this.model = model;
		value = obj;

		initButton();
	}

	/**
	 * Creates new button with specified palette buttom model.
	 *
	 * @param model Palette button model.
	 */
	public JPaletteButton(PaletteButtonModel model)
	{
		this.model = model;

		initButton();
	}

	private void initButton()
	{
		updateUI();

		setToolTipText( model.getToolTipText() );
		setMargin(new Insets(3,3,3,3));
	}

	/*
	 * ------- User interface --------
	 */

	/**
	 * String representing UI class that displays that component.
	 */
    public String getUIClassID()
	{
        return uiClassID;
    }

	/**
	 * Updates current look and feel UI class.
	 */
	public void updateUI()
	{
		setUI(new PaletteButtonUI());
		invalidate();
	}

	/*
	 * -------- Type and value ---------
	 */
	
	/**
	 * Gets model of this button.
	 */
	public PaletteButtonModel getPaletteModel()
	{
		return model;
	}

	/**
	 * Gets a value remembered by this button.
	 */
	public Object getValue()
	{
		return value;
	}

	/**
	 * Sets a value for this button. It supports property.
	 */
	public void setValue(Object o)
	{
		Object old = value;
		value = o;
		firePropertyChange(VALUE_PROPERTY, old, value);
	}

	/*
	 * ------- Icons ---------
	 */
	
	/**
	 * Gets icon from palette button model.
	 */
	public Icon getIcon()
	{
		return model.getIcon();
	}

	/**
	 * Sets icon rectangle.
	 */
	public void setIconRectangle(Rectangle rect)
	{
		iconRect = rect;
	}

	/**
	 * Gets icon rectangle.
	 */
	public Rectangle getIconRectangle()
	{
		return iconRect;
	}

	/*
	 * -------------- Members --------------
	 */

	private PaletteButtonModel model;

	private Object value;

	private Rectangle iconRect = null;
}
