/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet;

import java.awt.*;
import java.awt.event.ItemListener;
import java.awt.event.ItemEvent;
import java.beans.PropertyVetoException;
import javax.swing.*;
import javax.swing.table.*;
import java.util.*;

import org.jet.tracks.*;

/**
 * Developer palette.
 * This internal frame cannot be selected.
 *
 * @author Adam Domaski
 * @version 0.1 26/02/2004
 */
public class PaletteFrame extends JInternalFrame
{
	private Jet application;

	/**
	 * Creates toolbar frame.
	 */
	public PaletteFrame(Jet app)
	{
		super();
		application = app;

		resource = JetResources.getInstance();
		setTitle( resource.translate("SceneryToolbar.title") );

		setLayer( JLayeredPane.PALETTE_LAYER );

		// Przygotuj zawarto wizualn paska
		getContentPane().add( createPane() );
		initFrame();

		// Dodaj palet
		add(new TrackPalette(app));

		setSelected(false);
	}

	/**
	 * Sets default size and position for this frame.
	 */
	protected void initFrame()
	{
		setLocation(0, 0);
		Dimension dim = new Dimension(180, 180);
		setMinimumSize(dim);
		setSize(dim);
	}

	/**
	 * Creates contents for frame.
	 */
	private JPanel createPane()
	{
		JPanel pane = new JPanel();
		pane.setLayout(new BorderLayout());
		pane.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		JPanel tempPane = new JPanel();
		tempPane.setBorder(BorderFactory.createEmptyBorder(2, 2, 2, 2));
		jPaletteComboBox = new JComboBox();
		jPaletteComboBox.addItemListener(new SelectionHandler());
		tempPane.add(jPaletteComboBox);
		pane.add(tempPane, BorderLayout.NORTH);

		jControls = new JTable();
		prepareTable();
		JScrollPane scroll = new JScrollPane(jControls);

		pane.add(scroll);

		return pane;
	}

	private void prepareTable()
	{
		jControls.setTableHeader(null);
		jControls.setShowGrid(false);
		jControls.setIntercellSpacing(new Dimension(0, 0));		
		jControls.setDefaultRenderer( Object.class, new ButtonRenderer() );
		jControls.setDefaultEditor( Object.class, new ButtonEditor() );
		jControls.setRowHeight(28);
	}

	/**
	 * Registers new palette with palette frame.
	 */
	public void add(Palette palette)
	{
		paletteList.add(palette);

		jPaletteComboBox.addItem( palette.getName() );
	}

	/**
	 * This methods does not set selected flag.
	 * This internal frame is being prevented by selection.
	 */
	public void setSelected(boolean b)
	{
		try
		{
			if (b)
				return;
			super.setSelected(b);
		}
		catch (PropertyVetoException pve) {}
	}

	/*
	 * -------------- Handlers -------------
	 */
	
	class SelectionHandler implements ItemListener
	{
		public void itemStateChanged(ItemEvent ie)
		{
			Object item = ie.getItem();
			Iterator iterator = paletteList.iterator();
			while (iterator.hasNext())
			{
				Palette palette = (Palette) iterator.next();
				if ( palette.getName().equals((String) item) )
				{
					jControls.setModel( palette.getTableModel() );
					jControls.doLayout();
				}
			}
		}
	}

	/*
	 * ------------ Inner classes ----------
	 */
	
	/**
	 * This class is a renderer for table cell.
	 *
	 * @author Adam Domaski
	 * @version 0.1 6/07/2003
	 */
	public class ButtonRenderer implements TableCellRenderer
	{
		/**
		 * This method just maps value contained in cell to returning renderer.
		 *
		 * @param table Table.
		 * @param value Object contained in cell. In this case it is just
		 *  a button which is going to be renderer.
		 * @param isSelected Flag indicating selection of cell (<i>unused</i>).
		 * @param hasFocus Flag indicating focus of cell (<i>unused</i>).
		 * @param row Row number.
		 * @param col Column number.
		 * @return Renderer as a button.
		 */
		public Component getTableCellRendererComponent(JTable table, Object value,
			boolean isSelected, boolean hasFocus, int row, int col)
		{
			return (AbstractButton) value;
		}
	}

	/**
	 * This is a cell editor for table with toggle buttons.
	 * Editor is a toggle button mapped from concrete cell when editing
	 * is executed. After finished editing toggle button is mapped back to
	 * cell.
	 *
	 * @author Adam Domaski
	 * @version 0.1 6/07/2003
	 */
	public class ButtonEditor extends AbstractCellEditor
		implements TableCellEditor
	{
		private JToggleButton button;

		/**
		 * Builds editor
		 */
		public ButtonEditor()
		{
		}

		/**
		 * This method returns component played editor's role. This component
		 * is just a content of cell which is toggle button.
		 *
		 * @param table Editing table.
		 * @param value Content of the cell.
		 * @param isSelected A flag indicating selected cell.
		 * @param row Table row.
		 * @param col table column.
		 * @return Pressed toggle button.
		 */
		public Component getTableCellEditorComponent(JTable table, Object value,
			boolean isSelected, int row, int height)
		{
			table.repaint();
			button = (JToggleButton) value;
			button.setSelected(true);
			return button;
		}

		/**
		 * This method is executed after finishing cell editing.
		 * It returns a toggle button for edited cell.
		 *
		 * @return Toggle button.
		 */
		public Object getCellEditorValue()
		{
			return button;
		}
	}
	
	/*
	 * -------------- Members --------------
	 */

	/** I18n resource. */
	private JetResources resource;
	
	/** Palette list. */
	private java.util.List paletteList = new ArrayList();

	/** Combo box that displays registered palettes. */
	private JComboBox jPaletteComboBox;

	/** Table with palette controls. */
	private JTable jControls;
}
