/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.sce;

/**
 * Scenery event.
 * This is only interface so it is possible to create undoable events.
 *
 * @author Adam Domaski
 * @version 0.1 15/02/2004
 */
public interface SceneryEvent
{
	/*
	 * ---------- Constant ------------
	 */
	
	/** Insert event type. */
	public int INSERT = 1;

	/** Change event type. */
	public int CHANGE = 2;

	/** Remove event type. */
	public int REMOVE = 3;

	/**
	 * Returns scenery which sent this event.
	 *
	 * @return Scenery.
	 */
	public Scenery getScenery();

	/**
	 * Gets type of event.
	 * Event types are predefined values.
	 *
	 * @see #INSERT
	 * @see #CHANGE
	 * @see #REMOVE
	 */
	public int getType();

	/**
	 * Gets array of inserted elements.
	 *
	 * @return May return <code>null</code> when there was remove event.
	 */
	public Object[] getInserted();

	/**
	 * Gets array of removed elements.
	 *
	 * @return May return <code>null</code> when there was insert event.
	 */
	public Object[] getRemoved();
}