/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.sce;

/**
 * Scenery selection model tracks selection.
 * At now there is single selection support.
 *
 * @see ScenerySelectionListener
 * @see ScenerySelectionEvent
 * @author Adam Domaski
 * @version 0.1 14/09/2003
 */
public interface ScenerySelectionModel
{
	/**
	 * Adds scenery selection listener to listen to selection changes
	 * in model.
	 *
	 * @param l Listener to add.
	 * @see #removeScenerySelectionListener
	 */
	public void addScenerySelectionListener(ScenerySelectionListener l);
	
	/**
	 * Removes previously added a scenery selection listener.
	 *
	 * @param l Listener to remove.
	 * @see #addScenerySelectionListener
	 */
	public void removeScenerySelectionListener(ScenerySelectionListener l);

	/**
	 * Clears selection.
	 */
	public void clearSelection();

	/**
	 * Returns <code>true</code> if selection is empty.
	 */
	public boolean isSelectionEmpty();

	/**
	 * Sets selected object.
	 *
	 * @param obj Object to select.
	 */
	public void setSelectedObject(Object obj);

	/**
	 * Gets selected object.
	 *
	 * @return Selected object.
	 */
	public Object getSelectedObject();
}
