/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.tracks;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import javax.vecmath.*;

import org.jet.*;
import org.jet.sce.*;

/**
 * Custom arc track properties dialog.
 *
 * @author Adam Domaski
 * @version 0.1 11/03/2004
 */
public class CustomArcProperties extends TrackProperties
{
	public CustomArcProperties(Jet app)
	{
		super(app.getFrame());

		application = app;
		editable = true;

		resource = JetResources.getInstance();
		setTitle( resource.translate("CustomArcProperties.title") );

		setResizable(false);
		setDefaultCloseOperation(DISPOSE_ON_CLOSE);

		initialize();
		fillIn();

		pack();
		centerDialog();
		setVisible(true);
	}

	/**
	 * Initialize dialog controls.
	 */
	private void initialize()
	{
		JPanel pane = new JPanel();
		pane.setLayout(new BorderLayout());

		JPanel container = createContent();
		pane.add(container);

		JPanel controlPanel = new JPanel();		
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		controlPanel.setLayout(gb);
		controlPanel.setBorder(BorderFactory.createEmptyBorder(5, 5, 5, 5));

		// Actions
		Action okAction = new OKAction();
		Action caAction = new CancelAction();

		ActionMap actionMap = getLayeredPane().getActionMap();
		actionMap.put("close-it", caAction);
		actionMap.put("enter-it", okAction);

		JButton okButton = new JButton( resource.translate("TrackProperties.ok") );
		JButton caButton = new JButton( resource.translate("TrackProperties.cancel") );

		caButton.addActionListener(caAction);

		getRootPane().setDefaultButton(okButton);
		okButton.addActionListener(okAction);

		c.gridx = 0;
		c.gridy = 0;
		c.weightx = 1.0;
		c.anchor = GridBagConstraints.EAST;
		c.insets = new Insets(0, 0, 0, 5);
		gb.setConstraints(okButton, c);
		controlPanel.add(okButton);

		c.gridx = 1;
		c.gridy = 0;
		c.weightx = 0.0;
		c.insets = new Insets(0, 0, 0, 0);
		gb.setConstraints(caButton, c);
		controlPanel.add(caButton);

		normalize(okButton, caButton);
		okButton.requestFocus();

		pane.add(controlPanel, BorderLayout.SOUTH);

		getContentPane().add(pane);
	}

	/**
	 * Creates main content.
	 */
	private JPanel createContent()
	{
		JPanel jPanel;
		JLabel jLabel;
		JTextField jText;

		JPanel pane = new JPanel();
		pane.setName("TrackPropertiesPanel");
		pane.setBorder( BorderFactory.createEmptyBorder(1, 1, 1, 1) );
		
		JTabbedPane jTabbedPane = new JTabbedPane();
		jTabbedPane.setName("TrackPropertiesTabbedPane");

		JPanel jGeneralPanel = new JPanel();
		jGeneralPanel.setName("GeneralPanel");
		jGeneralPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		jGeneralPanel.setLayout(new GridLayout(3, 1, 0, 5));

		jPanel = new JPanel();
		GridBagLayout gb = new GridBagLayout();
		GridBagConstraints c = new GridBagConstraints();
		Insets insets = new Insets(1, 2, 1, 2);
		jPanel.setLayout(gb);
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		
		jLabel = new JLabel();
		jLabel.setName("NameLabel");
		jLabel.setText( resource.translate("TrackProperties.name") );
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.WEST;
		c.insets = insets;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createName();
		jText.setName("NameTextBox");
		jText.setColumns(10);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("LengthLabel");
		jLabel.setText( resource.translate("TrackProperties.length") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createLength();
		jText.setName("LengthTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("RadiusLabel");
		jLabel.setText( resource.translate("TrackProperties.radius") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRadius();
		jText.setName("RadiusTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("AngleLabel");
		jLabel.setText( resource.translate("CustomArcProperties.angle") );
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createDegreeFormatTextBox();
		jText.setName("AngleTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("WidthLabel");
		jLabel.setText( resource.translate("TrackProperties.width") );
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createWidth();
		jText.setName("WidthTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("FrictionLabel");
		jLabel.setText( resource.translate("TrackProperties.friction") );
		c.gridx = 2;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createFriction();
		jText.setName("FrictionTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TapLabel");
		jLabel.setText( resource.translate("TrackProperties.tap") );
		c.gridx = 4;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTap();
		jText.setName("TapTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 5;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("QualityLabel");
		jLabel.setText( resource.translate("TrackProperties.quality") );
		c.gridx = 4;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createQuality();
		jText.setName("QualityTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 5;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("DamageLabel");
		jLabel.setText( resource.translate("TrackProperties.damage") );
		c.gridx = 4;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createDamage();
		jText.setName("DamageTextBox");
		jText.setColumns(7);
		controls.putComponent(jText);
		c.gridx = 5;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jPanel.setFocusTraversalPolicy(new GeneralTabFocusTraversal());
		jGeneralPanel.add(jPanel);

		// Tor prawy / lewy
		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("TwirlLabel");
		jLabel.setText( resource.translate("CustomArcProperties.twirl") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		JPanel jRightArc = new JPanel();
		jRightArc.setBorder( BorderFactory.createEtchedBorder() );
		jRightArc.setLayout( new GridLayout(1, 2) );

		JRadioButton jRadioButton;
		ButtonGroup bGroup = new ButtonGroup();

		jRadioButton = new JRadioButton();
		jRadioButton.setName("LeftRadioButton");
		jRadioButton.setText( resource.translate("CustomArcProperties.left") );
		controls.putComponent(jRadioButton);
		bGroup.add(jRadioButton);
		jRightArc.add(jRadioButton);

		jRadioButton = new JRadioButton();
		jRadioButton.setName("RightRadioButton");
		jRadioButton.setText( resource.translate("CustomArcProperties.right") );
		controls.putComponent(jRadioButton);
		bGroup.add(jRadioButton);
		jRightArc.add(jRadioButton);

		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jRightArc, c);
		jPanel.add(jRightArc);

		jGeneralPanel.add(jPanel);

		// Koniec

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("EnviromentLabel");
		jLabel.setText( resource.translate("TrackProperties.enviroment") );
		c.gridx = 0;
		c.gridy = 0;
		c.insets = insets;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		JComboBox jComboBox = new JComboBox();
		jComboBox.setName("EnviromentComboBox");
		jComboBox.addItem(Track.BANK);
		jComboBox.addItem(Track.BRIDGE);
		jComboBox.addItem(Track.CANYON);
		jComboBox.addItem(Track.FLAT);
		jComboBox.addItem(Track.MOUNTAINS);
		jComboBox.addItem(Track.TUNNEL);
		controls.putComponent(jComboBox);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		jGeneralPanel.add(jPanel);

		jTabbedPane.addTab(resource.translate("TrackProperties.general"), jGeneralPanel);

		JPanel jTexturesPanel = new JPanel();
		jTexturesPanel.setName("TexturesPanel");
		jTexturesPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		jTexturesPanel.setLayout(new GridLayout(2, 1, 0, 5));

		/* Tekstury - grny panel */

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("VisibilityLabel");
		jLabel.setText( resource.translate("TrackProperties.visibility") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		JPanel jVisibilityPanel = new JPanel();
		jVisibilityPanel.setBorder( BorderFactory.createEtchedBorder() );
		jVisibilityPanel.setLayout( new GridLayout(1, 2) );

		bGroup = new ButtonGroup();
		jRadioButton = new JRadioButton();
		jRadioButton.setName("VisibleRadioButton");
		jRadioButton.setText( resource.translate("TrackProperties.visible") );
		controls.putComponent(jRadioButton);
		bGroup.add(jRadioButton);
		jVisibilityPanel.add(jRadioButton);

		jRadioButton = new JRadioButton();
		jRadioButton.setName("InvisibleRadioButton");
		jRadioButton.setText( resource.translate("TrackProperties.invisible") );
		controls.putComponent(jRadioButton);
		bGroup.add(jRadioButton);
		jVisibilityPanel.add(jRadioButton);

		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jVisibilityPanel, c);
		jPanel.add(jVisibilityPanel);
		
		jTexturesPanel.add(jPanel);

		/* Tekstury - dolny panel */

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("TextureLabel");
		jLabel.setText( resource.translate("TrackProperties.texture") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jComboBox = createTexture();
		jComboBox.setName("TextureComboBox");
		controls.putComponent(jComboBox);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		jLabel = new JLabel();
		jLabel.setName("TextureLengthLabel");
		jLabel.setText( resource.translate("TrackProperties.textureLength") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureLength();
		jText.setName("TextureLengthEditBox");
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("SecondTextureLabel");
		jLabel.setText( resource.translate("TrackProperties.secondTexture") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jComboBox = createSecondTexture();
		jComboBox.setName("SecondTextureComboBox");
		controls.putComponent( jComboBox );
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jComboBox, c);
		jPanel.add(jComboBox);

		jLabel = new JLabel();
		jLabel.setName("TextureWidthLabel");
		jLabel.setText( resource.translate("TrackProperties.textureWidth") );
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureWidth();
		jText.setName("TextureWidthTextBox");
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TextureHeightLabel");
		jLabel.setText( resource.translate("TrackProperties.textureHeight") );
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureHeight();
		jText.setName("TextureHeightTextBox");
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("TextureSlopeLabel");
		jLabel.setText( resource.translate("TrackProperties.textureSlope") );
		c.gridx = 2;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createTextureSlope();
		jText.setName("TextureSlopeTextBox");
		jText.setColumns(7);
		controls.putComponent( jText );
		c.gridx = 3;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jTexturesPanel.add(jPanel);

		jTabbedPane.addTab(resource.translate("TrackProperties.textures"), jTexturesPanel);

		/* Geometria */
		JPanel jGeometryPanel = new JPanel();
		jGeometryPanel.setName("GeometryPanel");
		jGeometryPanel.setBorder( BorderFactory.createEmptyBorder(3, 3, 3, 3) );
		jGeometryPanel.setLayout(new GridLayout(3, 1, 0, 5));

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("Point1Label");
		jLabel.setText( resource.translate("TrackProperties.point1") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Point1XTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point1YTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point1ZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Vector1Label");
		jLabel.setText( resource.translate("TrackProperties.vector1") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Vector1XTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector1YTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector1ZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Roll1Label");
		jLabel.setText( resource.translate("TrackProperties.roll1") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRoll();
		jText.setName("Roll1TextBox");
		jText.setColumns(6);
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jGeometryPanel.add(jPanel);

		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("Vector2Label");
		jLabel.setText( resource.translate("TrackProperties.vector2") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Vector2XTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector2YTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Vector2ZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Point2Label");
		jLabel.setText( resource.translate("TrackProperties.point2") );
		c.gridx = 0;
		c.gridy = 1;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createCoordinate();
		jText.setName("Point2XTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point2YTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 2;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jText = createCoordinate();
		jText.setName("Point2ZTextBox");
		jText.setColumns(6);
		jText.setEnabled( false );
		controls.putComponent(jText);
		c.gridx = 3;
		c.gridy = 1;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jLabel = new JLabel();
		jLabel.setName("Roll2Label");
		jLabel.setText( resource.translate("TrackProperties.roll2") );
		c.gridx = 0;
		c.gridy = 2;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createRoll();
		jText.setName("Roll2TextBox");
		jText.setColumns(6);
		jText.setEditable( editable );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 2;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jGeometryPanel.add(jPanel);

		// Velocity
		jPanel = new JPanel();
		jPanel.setBorder( BorderFactory.createCompoundBorder(
			BorderFactory.createEtchedBorder(),
			BorderFactory.createEmptyBorder(3, 3, 3, 3)) );
		gb = new GridBagLayout();
		jPanel.setLayout(gb);

		jLabel = new JLabel();
		jLabel.setName("VelocityLabel");
		jLabel.setText( resource.translate("CustomArcProperties.acceptableVelocity") );
		c.gridx = 0;
		c.gridy = 0;
		gb.setConstraints(jLabel, c);
		jPanel.add(jLabel);

		jText = createMediumFormatTextBox();
		jText.setName("VelocityTextBox");
		jText.setColumns(6);
		jText.setEditable( false );
		controls.putComponent(jText);
		c.gridx = 1;
		c.gridy = 0;
		gb.setConstraints(jText, c);
		jPanel.add(jText);

		jGeometryPanel.add(jPanel);

		jTabbedPane.addTab(resource.translate("TrackProperties.geometry"), jGeometryPanel);

		pane.add(jTabbedPane);

		return pane;
	}

	private void fillIn()
	{
		JTextField			jText;
		JFormattedTextField jFormat;
		JRadioButton		jRadioButton;
		JComboBox			jComboBox;

		Configuration config = application.config;

		jFormat = (JFormattedTextField) controls.getComponent("LengthTextBox");
		jFormat.setValue( config.get("CustomArcProperties.length") );
		double len = config.getDouble("CustomArcProperties.length");

		jFormat = (JFormattedTextField) controls.getComponent("RadiusTextBox");
		jFormat.setValue( config.get("CustomArcProperties.radius") );
		double rad = config.getDouble("CustomArcProperties.radius");

		jFormat = (JFormattedTextField) controls.getComponent("AngleTextBox");
		jFormat.setValue( new Double(Math.toDegrees(len / rad)) );

		jFormat = (JFormattedTextField) controls.getComponent("WidthTextBox");
		jFormat.setValue( config.get("CustomArcProperties.width") );

		jFormat = (JFormattedTextField) controls.getComponent("TapTextBox");
		jFormat.setValue( config.get("CustomArcProperties.tap") );

		jFormat = (JFormattedTextField) controls.getComponent("FrictionTextBox");
		jFormat.setValue( config.get("CustomArcProperties.friction") );

		jFormat = (JFormattedTextField) controls.getComponent("QualityTextBox");
		jFormat.setValue( config.get("CustomArcProperties.damage") );

		jFormat = (JFormattedTextField) controls.getComponent("DamageTextBox");
		jFormat.setValue( config.get("CustomArcProperties.quality") );

		boolean bool = ((Boolean) config.get("CustomArcProperties.right")).booleanValue();
		if (bool)
			jRadioButton = (JRadioButton) controls.getComponent("RightRadioButton");
		else
			jRadioButton = (JRadioButton) controls.getComponent("LeftRadioButton");
		jRadioButton.setSelected( true );

		jComboBox = (JComboBox) controls.getComponent("EnviromentComboBox");
		jComboBox.setSelectedItem( config.get("CustomArcProperties.enviroment") );

		bool = ((Boolean) config.get("CustomArcProperties.visibility")).booleanValue();
		if (bool)
			jRadioButton = (JRadioButton) controls.getComponent("VisibleRadioButton");
		else
			jRadioButton = (JRadioButton) controls.getComponent("InvisibleRadioButton");
		jRadioButton.setSelected( true );

		jComboBox = (JComboBox) controls.getComponent("TextureComboBox");
		jComboBox.setSelectedItem( config.get("CustomArcProperties.texture") );

		jFormat = (JFormattedTextField) controls.getComponent("TextureLengthEditBox");
		jFormat.setValue( config.get("CustomArcProperties.texLength") );

		jComboBox = (JComboBox) controls.getComponent("SecondTextureComboBox");
		jComboBox.setSelectedItem( config.get("CustomArcProperties.secondTexture") );

		jFormat = (JFormattedTextField) controls.getComponent("TextureWidthTextBox");
		jFormat.setValue( config.get("CustomArcProperties.texWidth") );

		jFormat = (JFormattedTextField) controls.getComponent("TextureHeightTextBox");
		jFormat.setValue( config.get("CustomArcProperties.texHeight") );

		jFormat = (JFormattedTextField) controls.getComponent("TextureSlopeTextBox");
		jFormat.setValue( config.get("CustomArcProperties.texSlope") );

		jFormat = (JFormattedTextField) controls.getComponent("Roll1TextBox");
		jFormat.setValue( config.get("CustomArcProperties.roll1") );

		jFormat = (JFormattedTextField) controls.getComponent("Roll2TextBox");
		jFormat.setValue( config.get("CustomArcProperties.roll2") );

		setConstraints();
		setListeners();
	}

	/**
	 * Sets the constraints for text fields.
	 */
	private void setConstraints()
	{
		JFormattedTextField jFormatText;
		
		jFormatText = (JFormattedTextField) controls.getComponent("Roll1TextBox");
		setConstraints(jFormatText, new Double(-Track.ROLL), new Double(Track.ROLL));
		jFormatText = (JFormattedTextField) controls.getComponent("Roll2TextBox");
		setConstraints(jFormatText, new Double(-Track.ROLL), new Double(Track.ROLL));

		jFormatText = (JFormattedTextField) controls.getComponent("AngleTextBox");
		setConstraints(jFormatText, new Double(0.0), new Double(90.0));
		jFormatText = (JFormattedTextField) controls.getComponent("RadiusTextBox");
		setConstraints(jFormatText, new Double(0.0));
	}

	/**
	 * Set document listeners.
	 */
	private void setListeners()
	{
		JTextField jText;

		jText = (JTextField) controls.getComponent("LengthTextBox");
		lengthListener = new LengthListener();
		jText.getDocument().addDocumentListener(lengthListener);

		jText = (JTextField) controls.getComponent("RadiusTextBox");
		radiusListener = new RadiusListener();
		jText.getDocument().addDocumentListener(radiusListener);
		if (application.config.getBoolean("TrackWiden"))
			jText.getDocument().addDocumentListener(new WidenListener());

		jText = (JTextField) controls.getComponent("AngleTextBox");
		angleListener = new AngleListener();
		jText.getDocument().addDocumentListener(angleListener);

		jText = (JTextField) controls.getComponent("Roll1TextBox");
		jText.getDocument().addDocumentListener(new RollListener(1));
		jText = (JTextField) controls.getComponent("Roll2TextBox");
		jText.getDocument().addDocumentListener(new RollListener(2));
	}

	/*
	 * ---------- Inner classes -----------
	 */
	
	abstract class DocumentAdapter implements DocumentListener
	{
		public void changedUpdate(DocumentEvent de)
		{
			act(de);
		}

		public void insertUpdate(DocumentEvent de)
		{
			act(de);
		}

		public void removeUpdate(DocumentEvent de)
		{
			act(de);
		}

		abstract protected void act(DocumentEvent de);		
	}

	class AngleListener extends DocumentAdapter
	{
		protected void act(DocumentEvent de)
		{
			JFormattedTextField jFormat;
			JFormattedTextField jRadius;
			JFormattedTextField jLength;

			jFormat = (JFormattedTextField) controls.getComponent("AngleTextBox");
			jRadius = (JFormattedTextField) controls.getComponent("RadiusTextBox");
			jLength = (JFormattedTextField) controls.getComponent("LengthTextBox");

			double angle = ((Number) jFormat.getValue()).doubleValue();

			double radius = ((Number) jRadius.getValue()).doubleValue();
			double length = angle / 180.0 * (Math.PI * radius);

			jLength.getDocument().removeDocumentListener( lengthListener );
			jLength.setValue( new Double(length) );
			jLength.getDocument().addDocumentListener( lengthListener );
		}
	}

	class LengthListener extends DocumentAdapter
	{
		protected void act(DocumentEvent de)
		{
			JFormattedTextField jAngle;
			JFormattedTextField jRadius;
			JFormattedTextField jLength;

			jAngle = (JFormattedTextField) controls.getComponent("AngleTextBox");
			jRadius = (JFormattedTextField) controls.getComponent("RadiusTextBox");
			jLength = (JFormattedTextField) controls.getComponent("LengthTextBox");

			double length = ((Number) jLength.getValue()).doubleValue();

			double radius = ((Number) jRadius.getValue()).doubleValue();
			double angle = length * 180.0 / (Math.PI * radius);

			jAngle.getDocument().removeDocumentListener(angleListener);
			jAngle.setValue( new Double(angle) );
			jAngle.getDocument().addDocumentListener(angleListener);			
		}
	}

	class RadiusListener extends DocumentAdapter
	{
		protected void act(DocumentEvent de)
		{
			JFormattedTextField jAngle;
			JFormattedTextField jRadius;
			JFormattedTextField jLength;
			JFormattedTextField jRoll1, jRoll2;
			JFormattedTextField jVelocity;

			jAngle = (JFormattedTextField) controls.getComponent("AngleTextBox");
			jRadius = (JFormattedTextField) controls.getComponent("RadiusTextBox");
			jLength = (JFormattedTextField) controls.getComponent("LengthTextBox");
			jRoll1 = (JFormattedTextField) controls.getComponent("Roll1TextBox");
			jRoll2 = (JFormattedTextField) controls.getComponent("Roll2TextBox");
			jVelocity = (JFormattedTextField) controls.getComponent("VelocityTextBox");

			double radius = ((Number) jRadius.getValue()).doubleValue();

			double angle = ((Number) jAngle.getValue()).doubleValue();
			double length =  angle / 180.0 * (Math.PI * radius);

			jLength.getDocument().removeDocumentListener(lengthListener);
			jLength.setValue( new Double(length) );
			jLength.getDocument().addDocumentListener(lengthListener);

			// Set acceptable velocity

			double roll1 = ((Number) jRoll1.getValue()).doubleValue();
			double roll2 = ((Number) jRoll2.getValue()).doubleValue();
			double roll = (roll1 + roll2) / 2 * 1000;

			double aua = application.
				config.getDouble("AdmissibleUnbalancedAcceleration");

			double vel = Math.sqrt((roll + 153*aua) / 11.8) * Math.sqrt(radius);

			jVelocity.setValue( new Double(vel) );
		}
	}

	class WidenListener extends DocumentAdapter
	{
		private final Double[] widths = {
			new Double(1.435), new Double(1.440),
			new Double(1.445), new Double(1.450) };

		protected void act(DocumentEvent de)
		{
			JFormattedTextField jRadius;
			JFormattedTextField jWidth;

			jRadius = (JFormattedTextField) controls.getComponent("RadiusTextBox");
			jWidth = (JFormattedTextField) controls.getComponent("WidthTextBox");

			double radius = ((Number) jRadius.getValue()).doubleValue();

			if (radius < 300.0 && radius >= 250.0)
			{
				jWidth.setValue( widths[1] );
			}
			else if (radius < 250.0 && radius >= 200.0)
			{
				jWidth.setValue( widths[2] );
			}
			else if (radius < 200.0)
			{
				jWidth.setValue( widths[3] );
			}
			else
			{
				jWidth.setValue( widths[0] );
			}
		}
	}

	class RollListener extends DocumentAdapter
	{
		public RollListener(int i)
		{
			if (i == 1)
			{
				jRoll1 = (JFormattedTextField) controls.getComponent("Roll1TextBox");
				jRoll2 = (JFormattedTextField) controls.getComponent("Roll2TextBox");
			}
			else if (i == 2)
			{
				jRoll2 = (JFormattedTextField) controls.getComponent("Roll1TextBox");
				jRoll1 = (JFormattedTextField) controls.getComponent("Roll2TextBox");
			}
		}

		protected void act(DocumentEvent de)
		{
			JFormattedTextField jRadius;
			JFormattedTextField jVelocity;

			jRadius = (JFormattedTextField) controls.getComponent("RadiusTextBox");
			jVelocity = (JFormattedTextField) controls.getComponent("VelocityTextBox");

			double roll1 = ((Number) jRoll1.getValue()).doubleValue();
			double roll2 = ((Number) jRoll2.getValue()).doubleValue();
			double commonRoll = (roll1 + roll2) / 2 * 1000;

			double radius = ((Number) jRadius.getValue()).doubleValue();
			double aua = application.
				config.getDouble("AdmissibleUnbalancedAcceleration");

			double vel = Math.sqrt((commonRoll + 153*aua) / 11.8) * Math.sqrt(radius);
			jVelocity.setValue( new Double(vel) );
		}

		private JFormattedTextField jRoll1;
		private JFormattedTextField jRoll2;
	}
	
	class OKAction extends AbstractAction
	{
		public void actionPerformed(ActionEvent ae)
		{
			double					d;
			int						i;
			JFormattedTextField		jFormat;
			JTextField				jText;
			JRadioButton			jButton;
			JComboBox				jComboBox;

			TrackFactory factory = TrackFactory.getInstance();

			jFormat = (JFormattedTextField) controls.getComponent("LengthTextBox");
			double length = ((Number) jFormat.getValue()).doubleValue();

			jFormat = (JFormattedTextField) controls.getComponent("RadiusTextBox");
			double radius = ((Number) jFormat.getValue()).doubleValue();

			jButton = (JRadioButton) controls.getComponent("RightRadioButton");
			if (!jButton.isSelected())
				radius = -radius;			

			Track track = factory.createArc(length, radius);

			// Uzupenij pozostae dane
			jText = (JTextField) controls.getComponent("NameTextBox");
			String name = jText.getText();
			if (name.equals(""))
				name = null;
			track.setName( name );

			// Width
			jFormat = (JFormattedTextField) controls.getComponent("WidthTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setWidth( d );

			// Tap
			jFormat = (JFormattedTextField) controls.getComponent("TapTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setTap( d );

			// Friction
			jFormat = (JFormattedTextField) controls.getComponent("FrictionTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setFriction( d );

			// Damage
			jFormat = (JFormattedTextField) controls.getComponent("DamageTextBox");
			i = ((Number) jFormat.getValue()).intValue();
			track.setDamage( i );

			// Quality
			jFormat = (JFormattedTextField) controls.getComponent("QualityTextBox");
			i = ((Number) jFormat.getValue()).intValue();
			track.setQuality( i );

			// Enviroment
			jComboBox = (JComboBox) controls.getComponent("EnviromentComboBox");
			track.setEnviroment( (String) jComboBox.getSelectedItem() );

			// Visibility
			jButton = (JRadioButton) controls.getComponent("VisibleRadioButton");
			track.setVisible( jButton.isSelected() );

			// Texture
			jComboBox = (JComboBox) controls.getComponent("TextureComboBox");
			track.setTexture( (String) jComboBox.getSelectedItem() );

			// Second texture
			jComboBox = (JComboBox) controls.getComponent("SecondTextureComboBox");
			track.setSecondTexture( (String) jComboBox.getSelectedItem() );

			// Texture length
			jFormat = (JFormattedTextField) controls.getComponent("TextureLengthEditBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setTextureLength(d);

			// Texture width
			jFormat = (JFormattedTextField) controls.getComponent("TextureWidthTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setTextureWidth(d);

			// Texture height
			jFormat = (JFormattedTextField) controls.getComponent("TextureHeightTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setTextureHeight(d);

			// Texture slope
			jFormat = (JFormattedTextField) controls.getComponent("TextureSlopeTextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setTextureSlope(d);

			// Roll 1
			jFormat = (JFormattedTextField) controls.getComponent("Roll1TextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setRoll1(d);

			// Roll 2
			jFormat = (JFormattedTextField) controls.getComponent("Roll2TextBox");
			d = ((Number) jFormat.getValue()).doubleValue();
			track.setRoll2(d);

			// Wsad to teraz do scenerii
			SceneryFrame frame = application.getTopmostFrame();
			if (frame != null)
			{
				JSceneryEditor editor = frame.getSceneryEditor();

				BasePoint basePoint = editor.getBasePoint();
				
				if (basePoint != null)
				{
				// Wykonaj obrt toru
				double alpha = - basePoint.getAlpha();
				double beta = - basePoint.getBeta();

				Matrix4d rotMatrix = new Matrix4d();
				rotMatrix.rotY(alpha);

				rotMatrix.transform( track.getPoint1() );
				rotMatrix.transform( track.getVector1() );
				rotMatrix.transform( track.getVector2() );
				rotMatrix.transform( track.getPoint2() );

				AxisAngle4d rotAxis = 
					new AxisAngle4d(track.getPoint2().z, 0.0, -track.getPoint2().x, beta);
				rotMatrix = new Matrix4d();
				rotMatrix.set(rotAxis);

				rotMatrix.transform( track.getPoint1() );
				rotMatrix.transform( track.getVector1() );
				rotMatrix.transform( track.getVector2() );
				rotMatrix.transform( track.getPoint2() );

				// wykonaj przesunicie toru
				Matrix4d traMatrix = new Matrix4d();
				traMatrix.setIdentity();
				traMatrix.setTranslation( new Vector3d(basePoint.getPoint()) );

				traMatrix.transform( track.getPoint1() );
				traMatrix.transform( track.getVector1() );
				traMatrix.transform( track.getVector2() );
				traMatrix.transform( track.getPoint2() );

				Scenery scenery = editor.getScenery();
				Object[] obj = new Object[1];
				obj[0] = track;
				scenery.insert(obj);

				// Zaznacz nowy obiekt
				editor.setSelectedObject(track);

				// Ustaw nowy punkt bazowy
				Point3d point = track.getPoint2();
				Vector3d resolved = null;

				if (!track.isStraight())
				{
					resolved = new Vector3d( track.getVector2() );
					resolved.negate();
				}
				else
				{
					Point3d p1 = track.getPoint1();
					resolved = new Vector3d();
					resolved.x = point.x - p1.x;
					resolved.y = point.y - p1.y;
					resolved.z = point.z - p1.z;
				}

				BasePoint nBasePoint = new BasePoint(point, resolved);
				editor.setBasePoint(nBasePoint);
				}
			}

			saveConfiguration(track);
			dispose();
		}

		private void saveConfiguration(Track track)
		{
			Configuration config = application.config;

			config.put("CustomArcProperties.length", new Double(track.getLength()));
			config.put("CustomArcProperties.radius", new Double(Math.abs(track.getRadius())));
			config.put("CustomArcProperties.width", new Double(track.getWidth()));
			config.put("CustomArcProperties.friction", new Double(track.getFriction()));
			config.put("CustomArcProperties.tap", new Double(track.getTap()));
			config.put("CustomArcProperties.damage", new Integer(track.getDamage()));
			config.put("CustomArcProperties.quality", new Integer(track.getQuality()));
			config.put("CustomArcProperties.enviroment", track.getEnviroment());
			config.put("CustomArcProperties.visibility", new Boolean(track.isVisible()));
			config.put("CustomArcProperties.texture", track.getTexture());
			config.put("CustomArcProperties.texLength", new Double(track.getTextureLength()));
			config.put("CustomArcProperties.secondTexture", track.getSecondTexture());
			config.put("CustomArcProperties.texWidth", new Double(track.getTextureWidth()));
			config.put("CustomArcProperties.texHeight", new Double(track.getTextureHeight()));
			config.put("CustomArcProperties.texSlope", new Double(track.getTextureSlope()));
			config.put("CustomArcProperties.roll1", new Double(track.getRoll1()));
			config.put("CustomArcProperties.roll2", new Double(track.getRoll2()));

			JRadioButton jRadio;
			jRadio = (JRadioButton) controls.getComponent("RightRadioButton");
			config.put("CustomArcProperties.right", new Boolean(jRadio.isSelected()));
		}

	}

	class CancelAction extends AbstractAction
	{
		public void actionPerformed(ActionEvent ae)
		{
			dispose();
		}
	}

	/*
	 * ------------- Members -------------
	 */
	
	/** Application. */
	private Jet application;

	/* Listeners. */
	private DocumentListener angleListener;
	private DocumentListener lengthListener;
	private DocumentListener radiusListener;
}
