/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003 2004  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package org.jet.tracks;

import java.awt.Point;
import java.awt.Rectangle;
import javax.swing.Icon;
import javax.swing.ImageIcon;

import org.jet.*;

/**
 * Model for flex track button.
 *
 * <p>Indexes are in range from 0 to <i>count</i>-1.</p>
 *
 * @author Adam Domaski
 * @version 0.1 21/06/2004
 */
public class FlexTrackButtonModel extends PaletteButtonModel
{
	/**
	 * Constructs new model for flex track.
	 *
	 * @param o Textual overview of the flex track.
	 */
	public FlexTrackButtonModel(String o)
	{
		overview = o;

		setSelectedHandler(0);
	}

	/**
	 * Gets handlers count in this model.
	 *
	 * @return Always returns 2.
	 */
	public int getHandlerCount()
	{
		return 2;
	}

	/**
	 * Returns point of handler with specified index. Returned point is in the icon coordinates.
	 *
	 * @param i Index of the handler.
	 * @return Point in icon coordinates.
	 */
	public Point handlerToPoint(int i)
	{
		if (i == 0)
			return point1;
		else if (i == 1)
			return point2;

		return null;
	}

	/**
	 * Returns index of handler with specified point. If there is no handler at this point, it returns -1.
	 *
	 * @param p Point in icon coordiantes.
	 * @return Index of handler or -1 when ther is no handler.
	 */
	public int pointToHandler(Point p)
	{
		Rectangle rect = new Rectangle(point1);
		rect.grow(3, 3);
		if (rect.contains(p))
			return 0;
		rect.setLocation(point2.x - 3, point2.y - 3);
		if (rect.contains(p))
			return 1;
		
		return -1;
	}

	/**
	 * Gets an icon of this model.
	 */
	public Icon getIcon()
	{
		return icon;
	}

	/**
	 * Gets tooltip text.
	 */
	public String getToolTipText()
	{
		return overview;
	}

	/*
	 * ------------ constants ---------------
	 */

	private static final Icon icon = new ImageIcon("images/jt_flex.png");

	private static final Point point1 = new Point(2, 2);
	private static final Point point2 = new Point(22, 8);

	private String overview;
}
