/*
 * jET - An alternative scenery editor for the electic locomotive simulator, MaSzyna
 * Copyright (C) 2003  Adam Domaski
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

package prv.doman.util;

/**
 * The <code>Token</code> class defines a token used by stream tokenizer.
 * Token contains a word and its type (integer value).
 *
 * @author Adam Domaski
 * @version 0.1 14/08/2003
 * @see Tokenizer
 */
public class Token
{
	/*
	 * --------------- Methods ---------------
	 */

	/**
	 * Gets a token's word.
	 * The word it is that what was read by tokenizer.
	 *
	 * @return The word.
	 */
	public String getWord()
	{
		return word;
	}

	/**
	 * Sets a word tothe token.
	 *
	 * param word A word.
	 */
	public void setWord(String word)
	{
		this.word = word;
	}

	/**
	 * Gets type of this token.
	 *
	 * @return Type of the token.
	 */
	public int getType()
	{
		return type;
	}

	/**
	 * Sets the type of token.
	 *
	 * @param type Type of the token.
	 */
	public void setType(int type)
	{
		this.type = type;		
	}

	/**
	 * Gets string representation of this object.
	 *
	 * @return The string representation.
	 */
	public String toString()
	{
		StringBuffer sb = new StringBuffer();

		sb.append("Token[");
		sb.append(getType());
		sb.append(',');
		sb.append(getWord());
		sb.append(" at ");
		sb.append(getLine());
		sb.append(',');
		sb.append(getColumn());
		sb.append(']');

		return sb.toString();
	}

	/**
	 * Gets the line of source where the token resides.
	 *
	 * @param Line number.
	 */
	public int getLine()
	{
		return line;
	}

	/**
	 * Sets the line number for this token.
	 *
	 * @param line Line number.
	 */
	public void setLine(int line)
	{
		this.line = line;
	}

	/**
	 * Gets an offset from beginning of line to the first
	 * character of the token.
	 *
	 * @return Offset of token within a line.
	 */
	public int getColumn()
	{
		return column;
	}

	/**
	 * Sets an offset for this token from beginning of line.
	 *
	 * @param offset Offset.
	 */
	public void setColumn(int offset)
	{
		column = offset;
	}

	/*
	 * ------------------ Members ---------------------
	 */
	
	/** A word. */
	private String word = null;

	/** The type of the word. */
	private int type;

	/** Line number. */
	private int line;

	/** Column number. */
	private int column;

	/*
	 * ----------------- Predefined -------------------
	 */
	
	/** Token type when a token is end-of-line. */
	public static final int TOKEN_EOL = -1;

	/** Token type when a token is regular word. */
	public static final int TOKEN_WORD = 0;

	/** Token type when a token is a number. */
	public static final int TOKEN_NUMBER = -2;

	/** Token type when a token is end-of-file. */
	public static final int TOKEN_EOF = -3;
}
